#
# ZBabelTag
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#
# USAGE:
#
# <dtml-babel src="language" dst="language" literal="1">
# TheText
# </dtml-babel>
#
# src: is the language that TheText written in
# dst: is the desired language that TheText is to be displayed in.
# literal: if set means that parsing of the block text will not occur
#
# You can use any designations you like for language.

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from DateTime.DateTime import DateTime
import Acquisition
import md5
import string

from ZBabelTower import ZBabelTower

class ZBabelTag(Acquisition.Implicit):
    """
    ZBabel Tag
    """

    name='babel'
    blockContinuations=()

    def __init__(self,blocks):

        tname, args, section = blocks[0]

        self.section=section
        #self.args=parse_params(args, src='en', dst='en', literal='')
        self.args=args=parse_let_params(args)

        # I stole this from dtml-let...
        for i in range(len(args)):
            name,expr = args[i]
            if expr[:1]=='"' and expr[-1:]=='"' and len(expr) > 1:
				# expr shorthand
                expr=expr[1:-1]
                try: args[i] = name, Eval(expr, expr_globals).eval
                except SyntaxError, v:
                    m,(huh,l,c,src) = v
                    raise ParseError, (
                        '<strong>Expression (Python) Syntax error</strong>:'
                        '\n<pre>\n%s\n</pre>\n' % v[0],
                        'let')

        print args
        # defaults
        self.literal=0

        # DEFAULT LANGUAGE IS EN -- override
        self.srclang='en'
        self.dstlang='en'


    def getarg_(self, name, defval):
        if self.args.has_key(name):
            return self.args[name]
        else:
            return defval

    def findBabelTower(self,root):

        while root is not None:
            if hasattr(root,'objectValues'):
                for o in root.objectValues():
                    try:
                        if (hasattr(o,'meta_type') and
                            o.meta_type=='ZBabel Tower'):
                            return o
                    except:
                        pass
            if hasattr(root,'aq_parent'):
                root=root.aq_parent
            else:
                root=None
        return None
                        
    def render(self,md):
        """ Do the Translation """
        thisText=''
        us=md.this

        d={};
        try:
            for name,expr in self.args:
                print name, expr
                if type(expr) is type(''):
                    try:
                        d[name]=md[expr]
                    except:
                        d[name]=expr
                else:
                    d[name]=expr(md)
        finally:
            pass

        # I know this is a bit well nearsighted, but, I'm setting
        # the default language to english.
        if d.has_key('src'):
            self.srclang=d['src']

        if d.has_key('dst'):
            self.dstlang=d['dst']

        if d.has_key('literal'):
            self.literal=1
            
        # Do we render the sub-blocks first and then translate?
        if self.literal:
            thisText=string.strip(self.section(md.this,md))
        else:
            thisText=string.strip(render_blocks(self.section.blocks,md))

        _Translator=self.findBabelTower(us)
        if not _Translator:
            return self.section(md.this,md)

        translation=_Translator.translate(thisText,self.srclang,
                                          self.dstlang,
                                          us.absolute_url())
        return translation
        
    __call__=render

def parse_let_params(text,
            result=None,
            tag='babel',
            parmre=regex.compile(
                '\([\0- ]*\([^\0- =\"]+\)=\([^\0- =\"]+\)\)'),
            qparmre=regex.compile(
                '\([\0- ]*\([^\0- =\"]+\)="\([^"]*\)\"\)'),
            **parms):

    result=result or []

    if parmre.match(text) >= 0:
        name=parmre.group(2)
        value=parmre.group(3)
        l=len(parmre.group(1))
    elif qparmre.match(text) >= 0:
        name=qparmre.group(2)
        value='"%s"' % qparmre.group(3)
        l=len(qparmre.group(1))
    else:
        if not text or not strip(text): return result
        raise ParseError, ('invalid parameter: "%s"' % text, tag)
    
    result.append((name,value))

    text=strip(text[l:])
    if text: return apply(parse_let_params,(text,result,tag),parms)
    else: return result

String.commands['babel'] = ZBabelTag
           
