#
# ZBabelTag
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
# Author: Andrew Milton <akm@theinternet.com.au>
#
# USAGE:
#
# <dtml-babel src="language" dst="language">
# TheText
# </dtml-babel>
#
# src: is the language that TheText written in
# dst: is the desired language that TheText is to be displayed in.
#
# You can use any designations you like for language.

from Globals import Persistent, HTMLFile, HTML, MessageDialog
from AccessControl.Role import RoleManager
import Acquisition, sys, string, types, mimetools
from OFS.Folder import Folder
import Globals
from cStringIO import StringIO
import md5, base64
from Products.ZCatalog.ZCatalog import ZCatalog
import urllib
from Products.ZSQLMethods.SQL import SQL



manage_addZBabelTowerForm=HTMLFile('manageAdd',globals())

def manage_addZBabelTower(self, id, connection, title=None, REQUEST=None):
    """ Add a Babel Tower """

    ob = ZBabelTower(id,connection)
    self._setObject(id,ob)
    
    if REQUEST:
        return self.manage_main(self,REQUEST)

class ZBabelTower(Folder,
                  Acquisition.Implicit,
                  RoleManager):
    ''' A Translation System '''

    meta_type='ZBabel Tower'

    manage_options=(
        {'label':'Contents', 'action':'manage_main'},
        {'label':'View', 'action':'index_html'},
        {'label':'Translate', 'action':'manage_Translate'},
        {'label':'Security', 'action':'manage_access'},
        {'label':'Undo', 'action':'manage_UndoForm'},
        {'label':'Find', 'action':'manage_findFrame', 'target':'manage_main'},
    )

    __ac_permission__ = (
        ('Manage ZBabel Towers', ('manage_main', 'manage',
                                  'manage_workspace',
                                  'manage_Translate',
                                  'reCatalog')),
        )

#    manage_main = Folder.manage_main
#    manage_workspace = Folder.manage_workspace

    manage_Translate = HTMLFile('Translations',globals())
    translatePage = HTMLFile('translatePage',globals())
    
    def __init__(self,id,connection, title=None):
        self.connection=connection
        self.id=id
        self.title=title
        self.registerSQLQueries()

    def registerSQLQueries(self):
        self.sqlFindMatches=SQL(
            'sqlFindMatches',
            'Find Matching Phrases',
            self.connection,
            'md5Hash:required',
            _sqlFindMatches)
        self.sqlFindMatches.manage_advanced(max_rows=1000,
                                           max_cache=1000,
                                           cache_time=60,
                                           class_name='',
                                           class_file='')
        self.sqlNewPhrase=SQL(
            'sqlNewPhrase',
            'Insert a New Phrase',
            self.connection,
            'lang:required md5Hash:required '+
            'renderPath:required renderText:required',
            _sqlNewPhrase)
        self.sqlGetAllPhrases=SQL(
            'sqlGetAllPhrases',
            'Get All Phrases',
            self.connection,
            '',
            _sqlGetAllPhrases)
        self.sqlGetPhrasesByPage=SQL(
            'sqlGetAllPhrases',
            'Get Phrases For on Page',
            self.connection,
            'renderPath srcLang dstLang',
            _sqlGetPhrasesByPage)
        self.sqlGetLanguages=SQL(
            'sqlGetLanguages',
            'Get Existing Languages',
            self.connection,
            '',
            _sqlGetLanguages)
        self.sqlGetPages=SQL(
            'sqlGetPages',
            'Get Pages with Translations',
            self.connection,
            '',
            _sqlGetPages)
        self.sqlUpdatePhrase=SQL(
            'sqlUpdatePhrase',
            'Update a Single Phrase',
            self.connection,
            'phrase_id:int:required renderText:required',
            _sqlUpdatePhrase)

        self.sqlGetPhrasesToUpdate=SQL(
            'sqlGetPhrasesToUpdate',
            'Force Translation Table Fill',
            self.connection,
            'srcLang:required dstLang:required',
            _sqlGetPhrasesToUpdate)

    def findTranslation(self, digest, srcText, src, dst,srcPath):
        matches=self.sqlFindMatches(md5Hash=digest)
        if matches:
            for hit in matches:
                if sqlattr(hit,'lang')==dst:
                    return sqlattr(hit,'renderText')

            # Insert a new phrase to be translated in
            # The destination language...
            self.sqlNewPhrase(lang=dst, md5Hash=digest,
                              renderPath=srcPath,
                              renderText=srcText)

            # Return our native text
            return srcText

        # This phrase is not registered at all
        # Register it...
        self.sqlNewPhrase(lang=src, md5Hash=digest, renderPath=srcPath,
                          renderText=srcText)

        # It's not registered and we also wanted a translation
        # Enter the destination translation
        if src != dst:
            self.sqlNewPhrase(lang=dst, md5Hash=digest,
                              renderPath=srcPath,
                              renderText=srcText)
        return srcText
        
    def translate(self, srcText, src, dst, srcPath):
        digest=md5.new()
        digest.update(srcText)
        digest=digest.digest()
        digest=string.strip(base64.encodestring(digest))
        matchText=self.findTranslation(digest,srcText,src,dst,srcPath)
        return matchText

    def updateTranslationDatabase(self,srcLang,dstLang,REQUEST=None):
        """ Force translation entries into a database """

        if srcLang == dstLang:
            if REQUEST:
                return self.manage_Translate(self,REQUEST)
            return None
        
        phrases=self.sqlGetPhrasesToUpdate(srcLang=srcLang,
                                             dstLang=dstLang)

        if phrases:
            for phrase in phrases:
                self.sqlNewPhrase(
                    lang=dstLang,
                    md5Hash=sqlattr(phrase,'md5Hash'),
                    renderPath=sqlattr(phrase,'renderPath'),
                    renderText=sqlattr(phrase,'renderText'))
        if REQUEST:
            return self.manage_Translate(self,REQUEST)

    def updatePhrases(self,REQUEST=None):
        """ Update all Phrases on a Page """
        if REQUEST:
            for formVar in REQUEST.form.keys():
                if formVar[:7]=='Phrase-':
                    crap,phraseId,lang=string.split(formVar,'-')
                    text=REQUEST.form[formVar]
                    self.sqlUpdatePhrase(phrase_id=int(phraseId),
                                         lang=lang,
                                         renderText=string.strip(text))
            return self.manage_Translate(self,REQUEST)
    

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name
        


_sqlFindMatches="""
SELECT *
FROM Phrases
WHERE md5Hash=<dtml-sqlvar md5Hash type=nb>
"""
_sqlNewPhrase="""
INSERT
INTO Phrases
VALUES (NULL,<dtml-sqlvar lang type=nb>,
        <dtml-sqlvar md5Hash type=nb>,
        <dtml-sqlvar renderPath type=nb>,
        <dtml-sqlvar renderText type=nb> )
"""

_sqlGetPages="""
SELECT DISTINCT renderPath
FROM Phrases
ORDER BY renderPath
"""

_sqlGetAllPhrases="""
SELECT *
FROM Phrases
ORDER BY renderPath
"""

_sqlGetPhrasesByPage="""
SELECT *
FROM Phrases
WHERE renderPath=<dtml-sqlvar renderPath type=nb>
AND
( lang=<dtml-sqlvar srcLang type=nb>
  OR  lang=<dtml-sqlvar dstLang type=nb>
  )
ORDER BY md5Hash
"""

_sqlGetLanguages="""
SELECT DISTINCT lang
FROM Phrases
"""

_sqlUpdatePhrase="""
UPDATE Phrases
SET renderText=<dtml-sqlvar renderText type=nb>
WHERE phrase_id=<dtml-sqlvar phrase_id type=int>
"""

_sqlGetPhrasesToUpdate="""
select Phrases.* FROM Phrases LEFT JOIN Phrases as b
ON Phrases.md5Hash = b.md5Hash AND Phrases.lang <> b.lang
WHERE b.md5Hash is NULL AND Phrases.lang=<dtml-sqlvar srcLang type=nb>
"""
